import _extends from "@babel/runtime/helpers/esm/extends";
import { useState, forwardRef } from 'react';
import { useDropzone } from 'react-dropzone';
import { getTarget } from '@s-ui/js/lib/react';
import cx from 'classnames';
import PropTypes from 'prop-types';
import useMount from '@s-ui/react-hooks/lib/useMount';
import useMergeRefs from '@s-ui/react-hooks/lib/useMergeRefs';
import { filterValidFiles, prepareFiles, loadInitialPhotos } from './fileTools';
import DragNotification from './DragNotification';
import DragState from './DragState';
import InitialState from './InitialState';
import PhotosPreview from './PhotosPreview';
import { ATOM_ICON_SIZES } from '@s-ui/react-atom-icon';
import { BASE_CLASS_NAME, DROPZONE_CLASS_NAME, DEFAULT_DRAG_DELAY_TIME, DEFAULT_IMAGE_ROTATION_DEGREES, DEFAULT_IMAGE_ASPECT_RATIO, DEFAULT_MAX_IMAGE_HEIGHT, DEFAULT_MAX_IMAGE_WIDTH, DEFAULT_FILE_TYPES_ACCEPTED, DEFAULT_MAX_FILE_SIZE_ACCEPTED, DEFAULT_NOTIFICATION_ERROR, DRAG_STATE_STATUS_REJECTED, ROTATION_DIRECTION, REJECT_FILES_REASONS } from './config';
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
import { Fragment as _Fragment } from "react/jsx-runtime";

var noop = function noop() {};

var MoleculePhotoUploader = /*#__PURE__*/forwardRef(function (_ref, forwardedRef) {
  var _cx;

  var _ref$acceptedFileType = _ref.acceptedFileTypes,
      acceptedFileTypes = _ref$acceptedFileType === void 0 ? DEFAULT_FILE_TYPES_ACCEPTED : _ref$acceptedFileType,
      _ref$acceptedFileMaxS = _ref.acceptedFileMaxSize,
      acceptedFileMaxSize = _ref$acceptedFileMaxS === void 0 ? DEFAULT_MAX_FILE_SIZE_ACCEPTED : _ref$acceptedFileMaxS,
      _ref$allowUploadDupli = _ref.allowUploadDuplicatedPhotos,
      allowUploadDuplicatedPhotos = _ref$allowUploadDupli === void 0 ? false : _ref$allowUploadDupli,
      addMorePhotosIcon = _ref.addMorePhotosIcon,
      addPhotoButtonColor = _ref.addPhotoButtonColor,
      addPhotoButtonDesign = _ref.addPhotoButtonDesign,
      addPhotoTextButton = _ref.addPhotoTextButton,
      addPhotoButtonSize = _ref.addPhotoButtonSize,
      addPhotoTextSkeleton = _ref.addPhotoTextSkeleton,
      _ref$callbackPhotosRe = _ref.callbackPhotosRejected,
      callbackPhotosRejected = _ref$callbackPhotosRe === void 0 ? noop : _ref$callbackPhotosRe,
      _ref$callbackPhotosUp = _ref.callbackPhotosUploaded,
      callbackPhotosUploaded = _ref$callbackPhotosUp === void 0 ? noop : _ref$callbackPhotosUp,
      callbackUploadPhoto = _ref.callbackUploadPhoto,
      deleteIcon = _ref.deleteIcon,
      _ref$disableScrollToB = _ref.disableScrollToBottom,
      disableScrollToBottom = _ref$disableScrollToB === void 0 ? false : _ref$disableScrollToB,
      _ref$dragDelay = _ref.dragDelay,
      dragDelay = _ref$dragDelay === void 0 ? DEFAULT_DRAG_DELAY_TIME : _ref$dragDelay,
      _ref$dragPhotosIcon = _ref.dragPhotosIcon,
      dragPhotosIcon = _ref$dragPhotosIcon === void 0 ? noop : _ref$dragPhotosIcon,
      dragPhotoTextInitialContent = _ref.dragPhotoTextInitialContent,
      dragPhotoDividerTextInitialContent = _ref.dragPhotoDividerTextInitialContent,
      dropPhotosHereText = _ref.dropPhotosHereText,
      errorCorruptedPhotoUploadedText = _ref.errorCorruptedPhotoUploadedText,
      errorFileExcededMaxSizeText = _ref.errorFileExcededMaxSizeText,
      errorFormatPhotoUploadedText = _ref.errorFormatPhotoUploadedText,
      errorInitialPhotoDownloadErrorText = _ref.errorInitialPhotoDownloadErrorText,
      _ref$infoIcon = _ref.infoIcon,
      infoIcon = _ref$infoIcon === void 0 ? noop : _ref$infoIcon,
      _ref$initialPhotos = _ref.initialPhotos,
      initialPhotos = _ref$initialPhotos === void 0 ? [] : _ref$initialPhotos,
      limitPhotosUploadedText = _ref.limitPhotosUploadedText,
      limitPhotosUploadedNotification = _ref.limitPhotosUploadedNotification,
      mainPhotoLabel = _ref.mainPhotoLabel,
      _ref$maxImageHeight = _ref.maxImageHeight,
      maxImageHeight = _ref$maxImageHeight === void 0 ? DEFAULT_MAX_IMAGE_HEIGHT : _ref$maxImageHeight,
      maxPhotos = _ref.maxPhotos,
      _ref$maxImageWidth = _ref.maxImageWidth,
      maxImageWidth = _ref$maxImageWidth === void 0 ? DEFAULT_MAX_IMAGE_WIDTH : _ref$maxImageWidth,
      notificationErrorFormatPhotoUploaded = _ref.notificationErrorFormatPhotoUploaded,
      _ref$outputImageAspec = _ref.outputImageAspectRatioDisabled,
      outputImageAspectRatioDisabled = _ref$outputImageAspec === void 0 ? false : _ref$outputImageAspec,
      _ref$outputImageAspec2 = _ref.outputImageAspectRatio,
      outputImageAspectRatio = _ref$outputImageAspec2 === void 0 ? DEFAULT_IMAGE_ASPECT_RATIO : _ref$outputImageAspec2,
      rejectPhotosIcon = _ref.rejectPhotosIcon,
      retryIcon = _ref.retryIcon,
      rotateIcon = _ref.rotateIcon,
      _ref$rotationDirectio = _ref.rotationDirection,
      rotationDirection = _ref$rotationDirectio === void 0 ? ROTATION_DIRECTION.counterclockwise : _ref$rotationDirectio,
      uploadingPhotosText = _ref.uploadingPhotosText,
      thumbIconSize = _ref.thumbIconSize;

  var _useState = useState([]),
      files = _useState[0],
      setFiles = _useState[1];

  var _useState2 = useState(Boolean(initialPhotos.length)),
      isLoading = _useState2[0],
      setIsLoading = _useState2[1];

  var _useState3 = useState(DEFAULT_NOTIFICATION_ERROR),
      notificationError = _useState3[0],
      setNotificationError = _useState3[1];

  var DEFAULT_FORMAT_TO_BASE_64_OPTIONS = {
    rotation: DEFAULT_IMAGE_ROTATION_DEGREES,
    outputImageAspectRatioDisabled: outputImageAspectRatioDisabled,
    outputImageAspectRatio: outputImageAspectRatio,
    maxImageHeight: maxImageHeight,
    maxImageWidth: maxImageWidth
  };
  var isPhotoUploaderEmpty = !files.length;

  var isPhotoUploaderFully = function isPhotoUploaderFully() {
    return files.length >= maxPhotos;
  };

  useMount(function () {
    if (initialPhotos.length) {
      loadInitialPhotos({
        initialPhotos: initialPhotos,
        defaultFormatToBase64Options: DEFAULT_FORMAT_TO_BASE_64_OPTIONS,
        setInitialDownloadError: function setInitialDownloadError() {
          setNotificationError({
            isError: true,
            text: errorInitialPhotoDownloadErrorText
          });
        },
        setFiles: setFiles,
        _callbackPhotosUploaded: _callbackPhotosUploaded,
        setIsLoading: setIsLoading
      });
    }
  });

  var _callbackPhotosUploaded = function _callbackPhotosUploaded(list) {
    var blobsArray = list.reduce(function (array, currentFile) {
      var blob = currentFile.blob,
          url = currentFile.url,
          isNew = currentFile.isNew,
          isModified = currentFile.isModified,
          hasErrors = currentFile.hasErrors,
          file = currentFile.file,
          preview = currentFile.preview,
          id = currentFile.id;
      array.push({
        blob: blob,
        url: url,
        isNew: isNew,
        isModified: isModified,
        hasErrors: hasErrors,
        file: file,
        previewUrl: preview,
        id: id
      });
      return [].concat(array);
    }, []);
    callbackPhotosUploaded(blobsArray);
  };

  var _onDropRejected = function _onDropRejected(rejectedFiles) {
    setNotificationError({
      isError: true,
      text: notificationErrorFormatPhotoUploaded
    });

    _scrollToBottom();

    var rejectedFilesWithReason = rejectedFiles.map(function (rejectedFile) {
      return {
        rejectedFile: rejectedFile,
        reason: REJECT_FILES_REASONS.fileType
      };
    });
    callbackPhotosRejected(rejectedFilesWithReason);
  };

  var _onDropAccepted = function _onDropAccepted(acceptedFiles) {
    setNotificationError(DEFAULT_NOTIFICATION_ERROR);
    if (isLoading) return false;

    if (isPhotoUploaderFully()) {
      setNotificationError({
        isError: true,
        text: limitPhotosUploadedNotification
      });

      _scrollToBottom();

      return false;
    }

    setIsLoading(true);
    var validFiles = filterValidFiles({
      files: files,
      filesToBeFiltered: acceptedFiles,
      acceptedFileMaxSize: acceptedFileMaxSize,
      handlePhotosRejected: callbackPhotosRejected,
      setMaxSizeError: function setMaxSizeError() {
        setNotificationError({
          isError: true,
          text: errorFileExcededMaxSizeText
        });

        _scrollToBottom();
      },
      setMaxPhotosError: function setMaxPhotosError() {
        setNotificationError({
          isError: true,
          text: limitPhotosUploadedNotification
        });
      },
      allowUploadDuplicatedPhotos: allowUploadDuplicatedPhotos,
      maxPhotos: maxPhotos
    });

    if (!validFiles.length) {
      setIsLoading(false);
      return false;
    }

    prepareFiles({
      handlePhotosRejected: callbackPhotosRejected,
      currentFiles: [].concat(files),
      newFiles: validFiles,
      defaultFormatToBase64Options: DEFAULT_FORMAT_TO_BASE_64_OPTIONS,
      errorCorruptedPhotoUploadedText: errorCorruptedPhotoUploadedText,
      setCorruptedFileError: function setCorruptedFileError(errorText) {
        setNotificationError({
          isError: true,
          text: errorText
        });
      },
      setFiles: setFiles,
      setIsLoading: setIsLoading,
      _scrollToBottom: _scrollToBottom,
      callbackUploadPhoto: callbackUploadPhoto,
      _callbackPhotosUploaded: _callbackPhotosUploaded
    });
  };

  var _useDropzone = useDropzone({
    noClick: isPhotoUploaderFully(),
    noKeyboard: isPhotoUploaderFully(),
    accept: acceptedFileTypes,
    onDropAccepted: function onDropAccepted(acceptedFiles) {
      return _onDropAccepted(acceptedFiles);
    },
    onDropRejected: function onDropRejected(rejectedFiles) {
      return _onDropRejected(rejectedFiles);
    }
  }),
      getRootProps = _useDropzone.getRootProps,
      getInputProps = _useDropzone.getInputProps,
      isDragActive = _useDropzone.isDragActive,
      isDragAccept = _useDropzone.isDragAccept,
      isDragReject = _useDropzone.isDragReject,
      dropzoneInputRef = _useDropzone.inputRef;

  var inputRef = useMergeRefs(dropzoneInputRef, forwardedRef);
  var dropzoneClassName = cx(DROPZONE_CLASS_NAME, (_cx = {}, _cx[DROPZONE_CLASS_NAME + "--disabled"] = isPhotoUploaderFully(), _cx[DROPZONE_CLASS_NAME + "--empty"] = isPhotoUploaderEmpty, _cx));
  var container = getTarget(document.querySelector("." + BASE_CLASS_NAME));

  var _scrollToBottom = function _scrollToBottom() {
    if (!disableScrollToBottom) {
      var bounding = container.getBoundingClientRect();

      if (bounding.bottom > (window.innerHeight || document.documentElement.clientHeight)) {
        container.scrollIntoView({
          behavior: 'smooth',
          block: 'end'
        });
      }
    }
  };

  return /*#__PURE__*/_jsx(_Fragment, {
    children: /*#__PURE__*/_jsxs("div", {
      className: BASE_CLASS_NAME,
      children: [/*#__PURE__*/_jsxs("div", _extends({}, getRootProps({
        className: dropzoneClassName
      }), {
        children: [/*#__PURE__*/_jsx("input", _extends({}, getInputProps(), {
          ref: inputRef
        })), isPhotoUploaderEmpty && !isDragActive && /*#__PURE__*/_jsx(InitialState, {
          buttonColor: addPhotoButtonColor,
          buttonDesign: addPhotoButtonDesign,
          buttonText: addPhotoTextButton,
          buttonSize: addPhotoButtonSize,
          icon: dragPhotosIcon(),
          text: dragPhotoTextInitialContent,
          dividerText: dragPhotoDividerTextInitialContent
        }), !isPhotoUploaderEmpty && /*#__PURE__*/_jsx(PhotosPreview, {
          _callbackPhotosUploaded: _callbackPhotosUploaded,
          _scrollToBottom: _scrollToBottom,
          addMorePhotosIcon: addMorePhotosIcon,
          addPhotoTextSkeleton: addPhotoTextSkeleton,
          callbackUploadPhoto: callbackUploadPhoto,
          defaultFormatToBase64Options: DEFAULT_FORMAT_TO_BASE_64_OPTIONS,
          deleteIcon: deleteIcon,
          dragDelay: dragDelay,
          errorInitialPhotoDownloadErrorText: errorInitialPhotoDownloadErrorText,
          files: files,
          isPhotoUploaderFully: isPhotoUploaderFully,
          mainPhotoLabel: mainPhotoLabel,
          outputImageAspectRatioDisabled: outputImageAspectRatioDisabled,
          rejectPhotosIcon: rejectPhotosIcon,
          rotateIcon: rotateIcon,
          rotationDirection: rotationDirection,
          retryIcon: retryIcon,
          setFiles: setFiles,
          setIsLoading: setIsLoading,
          setNotificationError: setNotificationError,
          thumbIconSize: thumbIconSize
        }), isDragAccept && !isPhotoUploaderFully() && !isLoading && /*#__PURE__*/_jsx(DragState, {
          icon: dragPhotosIcon(),
          text: dropPhotosHereText
        }), isDragAccept && isPhotoUploaderFully() && /*#__PURE__*/_jsx(DragState, {
          icon: rejectPhotosIcon(),
          status: DRAG_STATE_STATUS_REJECTED,
          text: limitPhotosUploadedText
        }), isDragAccept && !isPhotoUploaderFully() && isLoading && /*#__PURE__*/_jsx(DragState, {
          icon: rejectPhotosIcon(),
          status: DRAG_STATE_STATUS_REJECTED,
          text: uploadingPhotosText
        }), isDragReject && /*#__PURE__*/_jsx(DragState, {
          icon: rejectPhotosIcon(),
          status: DRAG_STATE_STATUS_REJECTED,
          text: errorFormatPhotoUploadedText
        })]
      })), /*#__PURE__*/_jsx(DragNotification, {
        icon: infoIcon(),
        onCloseCallback: function onCloseCallback() {
          return setNotificationError(DEFAULT_NOTIFICATION_ERROR);
        },
        show: notificationError.isError,
        text: notificationError.text
      })]
    })
  });
});
MoleculePhotoUploader.displayName = 'MoleculePhotoUploader';
export default MoleculePhotoUploader;
export { ROTATION_DIRECTION as MoleculePhotoUploaderRotationDirection };